# -*- coding: utf-8 -*-

#######################################################################
# Copyright © 2007-2009 Yuri Malheiros.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published
# the Free Software Foundation; version 2 only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#######################################################################

"""
This module implements a class for responsible for manage all
opened text files of TextFlow.
"""

import gtk
import gtksourceview2
import gnomevfs
import gobject
import os
import pango
import gettext

from TextFlow.core.preferencesmanager import PreferencesManager
from TextFlow.core.triggermanager import TriggerManager
from TextFlow.core.languagemanager import LanguageManager
from TextFlow.core import constants
from TextFlow.com.search import Search

from TextFlow.widgets.tfsourceview import TFSourceView
from TextFlow.widgets.filedialog import SaveFileDialog as SaveFileDialog
from TextFlow.widgets.closealertdialog import CloseAlertDialog
from TextFlow.widgets.errordialog import ErrorDialog
from TextFlow.widgets.changeencodingdialog import ChangeEncodingDialog
from TextFlow.widgets.closebutton import CloseButton

_ = gettext.gettext

class DocumentManager(gtk.Notebook):
    """
    This class extend the gtk.Notebook widget with functions
    to manipulate opened files.
    """

    __gsignals__ = { "open-file" : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
                                 (gobject.TYPE_STRING,)),
                     "save-file" :  (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
                                 (gobject.TYPE_STRING,)),
                     "close-file" :  (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
                                 (gobject.TYPE_STRING,))}

    def __init__(self, interface):
        """
        Constructor.
        """
        super(DocumentManager, self).__init__()
        
        self.interface = interface
        self.main_window = interface.main_window
        
        self.preferences_manager = PreferencesManager()
        self.trigger_manager = TriggerManager(self)
        self.language_manager = LanguageManager(self, interface.menuitem_language)
        
        self.search_functions = Search(self)
        
        self.set_scrollable(True)
        self.connect("switch-page", self.change_tab)
        
        self.__get_color_styles()
        self.doing_sticky = False
    
    #################### Public Methods ####################
    
    def open_tab(self, text, file_uri="", encode=""):
        """
        Open a new tab in the DocumentManager
        
        @param text: Tab label text.
        @type text: A string.
        
        @param file_uri: A file that will be displayed in TFSourceView.
        @type file_uri: A string.
        """
        
        tab_box = gtk.HBox(False, 4)
        tab_label = gtk.Label(text)
        tab_button = CloseButton()
        
        tab_box.pack_start(tab_label, True, True, 0)
        tab_box.pack_start(tab_button, False, False, 0)
        tab_box.show_all()
        
        swindow = gtk.ScrolledWindow()
        swindow.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_ALWAYS)
        
        # Verifying if the file is already open
        if file_uri != "":
            tab_number = self.get_n_pages()
            for number in range(tab_number):
                if self.get_nth_page(number).get_child().file_uri == file_uri:
                    self.set_current_page(number)
                    return
        
        try:
            view = self.__create_view(file_uri, encode)
        except UnicodeDecodeError:
            dialog_encoding = ChangeEncodingDialog()
            button_clicked = dialog_encoding.run()
            
            if button_clicked == gtk.RESPONSE_DELETE_EVENT:
                dialog_encoding.destroy()
                return
            elif button_clicked == 1:
                encoding = dialog_encoding.get_encoding()
                self.open_tab(text, file_uri, encoding)
                dialog_encoding.destroy()
                return
            elif button_clicked == 2:
                dialog_encoding.destroy()
                return
        else:
            swindow.add(view)
            swindow.show_all()
            
            child = self.get_nth_page(0)
            
            if child != None:
                v_child = child.get_child()
            
            if self.get_n_pages() == 1:
                if v_child.get_file_uri() == "" and v_child.get_text() == "":
                    if view.get_file_uri() != "":
                        self.__remove_tab(view, 0)
            elif self.get_n_pages() == 0:
                self.interface.statusbar.show_info()
            
            new_page_num = self.page_num(swindow)
            
            if file_uri != "":
                if self.preferences_manager.get_value("open_save/auto_save"):
                    self.add_to_autosave(view)
            
            self.append_page(swindow, tab_box)
            self.set_tab_reorderable(swindow, True)
            self.set_current_page(new_page_num)
            tab_button.connect('clicked', self.close_tab_button, swindow)
            view.buffer.connect('changed', self.buffer_change, view, tab_label)
            self.emit("open-file", file_uri)
            view.grab_focus()

    def close_tab(self, swindow):
        """
        Close a tab in the DocumentManager
        
        @param swindow: A ScrolledWindow.
        @type swindow: A ScrolledWindow object.
        """
        view = swindow.get_child()
        
        if not view.is_updated():
            dialog = CloseAlertDialog()
            
            button_clicked = dialog.run()
            
            if button_clicked == gtk.RESPONSE_DELETE_EVENT:
                dialog.destroy()
                return
            elif button_clicked == 1:
                dialog.destroy()
            elif button_clicked == 2:
                dialog.destroy()
                return
            elif button_clicked == 3:
                file_uri = view.get_file_uri()
                if file_uri != "":
                    self.save_file_tab(file_uri)
                else:
                    file_save = SaveFileDialog(constants.MESSAGE_0002, None,
                                               "", "*", False)
                    files = file_save.run()
                    file_save.destroy()
            
                    if len(files):
                        self.save_file_tab(files[0])
                    
                
                self.emit("close-file", file_uri)
                dialog.destroy()
            
        if view.file_uri != "" and view.autosave:
            self.remove_from_autosave(view)
        
        #Close tab frees memory correctly?
        page = self.page_num(swindow)
        self.__remove_tab(view, page)
        
        if self.get_n_pages() == 0:
            self.open_tab(constants.MESSAGE_0001)
        
    def open_file_tab(self, new_file, encode=""):
        """
        Open a new tab with a file associated.
        
        @param new_file: A file that will be displayed in the new tab.
        @type new_file: A string.
        """
        self.open_tab(os.path.basename(new_file), new_file, encode)
    
    def save_file_tab(self, save_file, show_error=True):
        """
        Save the text of active tab in a file.
        
        @param save_file: A file that will be save.
        @type save_file: A string.
        """
        view = self.get_active_view()
        self.__save_view(view, save_file, show_error)
    
    def get_active_view(self):
        """
        Get the active TFSourceView
        
        @return: The active TFSourceView.
        @rtype: A TFSourceView object.
        """
        current_page = self.get_current_page()
        
        if current_page == -1:
            return None
        else:
            return self.get_nth_page(current_page).get_child()
    
    def store_tabs(self):
        """
        Save file uri and the encode of all tabs in a text file.
        """
        store_file_uri = os.path.join(constants.HOME, ".textflow", "stored_tabs")
        store_file = open(store_file_uri, "w")
        
        num_pages = self.get_n_pages()
        
        for i in range(num_pages):
            view = self.get_nth_page(i).get_child()
            if view.file_uri != "":
                store_file.write(view.file_uri + "\n" + view.encode + "\n")
        
        store_file.close()
        
    def open_stored_tabs(self):
        """
        Open tabs according stored_tabs file data.
        """
        
        store_file_uri = os.path.join(constants.HOME, ".textflow", "stored_tabs")
        try:
            store_file = open(store_file_uri, "r")
        except IOError:
            return
        else:
            lines = store_file.readlines()
            cont = 0
            while cont < len(lines):
                try:
                    if os.path.exists(lines[cont][:-1]):
                        self.open_file_tab(lines[cont][:-1], lines[cont+1][:-1])
                except:
                    pass
                
                # jump 2 lines
                cont += 2
            
    def change_encode(self, encode):
        """
        Change the text encode of current view
        
        @param encode: Encode name.
        @type encode: A String.
        """
        
        page_num = self.get_current_page()
        
        if page_num != -1:
            swindow = self.get_nth_page(page_num)
            view = swindow.get_child()

            if view.encode != encode:
                file_uri = view.file_uri
                
                if file_uri != "":
                    if not view.is_updated():
                        dialog = CloseAlertDialog()
                        dialog.set_markup(constants.MESSAGE_0014)

                        button_clicked = dialog.run()

                        if button_clicked == gtk.RESPONSE_DELETE_EVENT:
                            dialog.destroy()
                            return
                        elif button_clicked == 1:
                            dialog.destroy()
                        elif button_clicked == 2:
                            dialog.destroy()
                            return
                        elif button_clicked == 3:
                            self.save_file_tab(file_uri)
                           
                            dialog.destroy()
                                                       
                        page = self.page_num(swindow)
                        self.__remove_tab(view, page)
                        self.open_file_tab(file_uri, encode)
                    else:
                        page = self.page_num(swindow)
                        self.__remove_tab(view, page)
                        self.open_file_tab(file_uri, encode)
                        
                    new_page_num = self.get_current_page()
                    swindow = self.get_nth_page(new_page_num)
                    self.reorder_child(swindow, page_num)
    
    def add_to_autosave(self, view):
        """
        Add a view to auto save.
        
        @param view: the view added to auto save.
        @type view: TFSourceView.
        """
        minutes = self.preferences_manager.get_value("open_save/auto_save_minutes")
        view.autosave = gobject.timeout_add(minutes*60000, self.__save_view, view,
                                            view.file_uri, priority=gobject.PRIORITY_LOW)

    def remove_from_autosave(self, view):
        """
        Remove a view from auto save.
        
        @param view: the view added to auto save.
        @type view: TFSourceView.
        """
        gobject.source_remove(view.autosave)
        view.autosave = False

    def set_style(self, view, value):
        """
        Set the color style used by a TFSourceView.
        
        @param view: The TFSourceView.
        @type view: A TFSourceView object.
        
        @param value: The font name.
        @type value: A string.
        """
        
        scheme = self.style_manager.get_scheme(value)
        buffer = view.buffer
        
        if scheme != None:
            buffer.set_style_scheme(scheme)
            
            style_scheme = buffer.get_style_scheme()
            style = style_scheme.get_style("search-match")
            style_sticky = style_scheme.get_style("def:note")
            
            # Set search match color
            if style != None:
                foreground = style.get_property("foreground")
                background = style.get_property("background")
            else:
                foreground = "black"
                background = "lightblue"

            if style_sticky != None:
                foreground_sticky = style_sticky.get_property("foreground")
                background_sticky = style_sticky.get_property("background")
            else:
                foreground_sticky = "black"
                background_sticky = "#ff6100"
                
            tag_table = buffer.get_tag_table()
            tag = tag_table.lookup("searchmatch")
            tag_sticky = tag_table.lookup("sticky")
            
            if foreground != None:
                tag.set_property("foreground", foreground)
                tag_sticky.set_property("foreground", foreground_sticky)
            else:
                tag.set_property("foreground", "black")
                tag_sticky.set_property("foreground", "black")
            
            if background != None:
                tag.set_property("background", background)
                tag_sticky.set_property("background", background_sticky)
            
    def set_text_wrapping(self, view, value):
        """
        Enable or disable text wrapping in a TFSourceView.
        
        @param view: The TFSourceView.
        @type view: A TFSourceView object.
        
        @param value: True to enable text wrapping and False to disable.
        @type value: A boolean.
        """
        if value:
            view.set_wrap_mode(gtk.WRAP_WORD)
        else:
            view.set_wrap_mode(gtk.WRAP_NONE)
    
    def undo(self):
        """
        Undo a actionr of current view.
        """
        
        view = self.get_active_view()
        buffer = view.buffer
        can_undo = buffer.can_undo()
        
        if view != None and can_undo:
            buffer.undo()
            
            can_undo = buffer.can_undo()
            
            if not can_undo and view.is_updated():
                child = self.get_nth_page(self.get_current_page())
                label_box = self.get_tab_label(child)
                label = label_box.get_children()[0]
                label.set_label(label.get_label()[1:])
                view.modified = False
                
        
    #################### Private Methods ####################
    
    def __get_color_styles(self):
        """
        This method gets all available color styles.
        """
        styles_dir = os.getenv("HOME") + "/.textflow/colors"
        self.style_manager = gtksourceview2.StyleSchemeManager()
        self.style_manager.prepend_search_path(styles_dir)
    
    def __create_view(self, file_uri, encode):
        """
        Create a TFSourceView and set some properties.
        
        @param file_uri: A file that will be displayed in TFSourceView.
        @type file_uri: A string
        
        @return: The TFSourceView created.
        @rtype: A TFSourceView object.
        """
        buffer = gtksourceview2.Buffer()
        pm = self.preferences_manager
        
        #Color stuff
        buffer.create_tag("searchmatch", background="lightblue")
        buffer.create_tag("sticky", background="#ff6100")
        buffer.set_highlight_syntax(True)
        
        view = TFSourceView(buffer, file_uri, encode)
        
        #Getting view language id
        language = view.buffer.get_language()
        
        if language != None:
            language_id = language.get_id()
            self.language_manager.change_mode(language_id)
            language_configs = self.language_manager.loaded_configs[language_id]
            view.set_tab_width(language_configs["tab-size"])
        else:
            self.language_manager.change_mode(None)
            view.set_tab_width(pm.get_value("indentation/tab_width"))
        
        # Define view encode to default encoding 
        if view.encode == "":
            view.encode = pm.get_value("open_save/encoding")
        
        #applying properties
        view.modify_font(pango.FontDescription(pm.get_value("font")))
        view.set_insert_spaces_instead_of_tabs(pm.get_value("indentation/use_spaces"))
        view.set_auto_indent(pm.get_value("indentation/automatic"))
        view.set_show_right_margin(pm.get_value("right_margin/display"))
        view.set_right_margin_position(pm.get_value("right_margin/position"))
        view.set_show_line_numbers(pm.get_value("line_numbers"))
        view.set_highlight_current_line(pm.get_value("highlight_current_line"))
        self.set_text_wrapping(view, pm.get_value("text_wrapping"))
        view.buffer.set_highlight_matching_brackets(pm.get_value("brackets_matching"))
        view.set_smart_home_end(False)
        self.set_style(view, pm.get_value("color_style"))
        
        view.connect('key-press-event', self.pre_key_press)
        view.connect('key-release-event', self.key_release)
        view.connect("drag-motion", self.drag_motion)
        view.connect("drag-drop", self.drag_drop)
        view.connect("drag-data-received", self.drag_data_received)
        view.drag_dest_set(gtk.DEST_DEFAULT_ALL, [("text/uri-list", 0, 1)],
                           gtk.gdk.ACTION_COPY)
        view.buffer.connect("notify::cursor-position", self.cursor_moved, view,
                            self.interface.statusbar)
                            
        view.show()
        return view
    
    def __remove_tab(self, view, page):
        view.word_complete.stop_indexer()
        self.remove_page(page)
    
    def __save_view(self, view, save_file, show_error=True):
        
        try:
            text = view.get_text().encode(view.encode)
        except UnicodeEncodeError:
            error_dialog = ErrorDialog(constants.MESSAGE_0018,
                                       constants.MESSAGE_0025)
            button_clicked = error_dialog.run()
            
            if button_clicked == gtk.RESPONSE_DELETE_EVENT:
                error_dialog.destroy()
                return
            elif button_clicked == 1:
                error_dialog.destroy()
            return
        
        try:
            arquivo = open(save_file, "w")
        except IOError:
            if show_error:
                view.permission = "Read Only"
                self.interface.statusbar.permission_label.set_text(view.permission)
                error_dialog = ErrorDialog(constants.MESSAGE_0018,
                                           constants.MESSAGE_0019)
                button_clicked = error_dialog.run()
                
                if button_clicked == gtk.RESPONSE_DELETE_EVENT:
                    error_dialog.destroy()
                    return
                elif button_clicked == 1:
                    error_dialog.destroy()
            else:
                return
        else:
            
            
            line_ending = self.preferences_manager.get_value("open_save/line_ending")
            
            if line_ending == "CR LF":
                text = text.replace("\n","\r\n")

            if self.preferences_manager.get_value("open_save/save_copy") == True:
                arquivo_copy = open(save_file + '~', "w")
                arquivo_copy.write(text)                
                arquivo_copy.close()
            
            arquivo.write(text)
            arquivo.close()
            self.emit("save-file", save_file)
            
            if view.file_uri == "":
                if self.preferences_manager.get_value("open_save/auto_save"):
                    self.add_to_autosave(view)
            
            if view.get_file_uri() != save_file:
                view.set_file_uri(save_file)
                
                
            
            #Getting view language id
            language = view.buffer.get_language()
            
            if language != None:
                language_id = language.get_id()
                
                if (self.language_manager.language != language_id):
                    self.language_manager.change_mode(language_id)
                    language_configs = self.language_manager.loaded_configs[language_id]
                    view.set_tab_width(language_configs["tab-size"])
            else:
                self.language_manager.change_mode(None)
                #view.set_tab_width(self.preferences_manager.get_value("indentation/tab_width"))
            
            self.interface.statusbar.combobutton_tab_size.set_label(constants.MESSAGE_0026 + " " + str(view.get_tab_width()))
            
            
            current_page = self.get_current_page()
            current_child = self.get_nth_page(current_page)
            tab_label = self.get_tab_label(current_child)
            children = tab_label.get_children()
            
            f_list = save_file.split("/")
            file_name = f_list.pop()
            direc = " (" + "/".join(f_list) + ")"
            
            children[0].set_label(file_name)
            
            self.main_window.set_title(str(file_name) + direc)
    #        begin = view.buffer.get_start_iter()
    #        end = view.buffer.get_end_iter()
    #         self.search_functions.update_matches(begin, end)
            
            view.modified = False
            view.permission = "Writable"
            self.interface.statusbar.permission_label.set_text(view.permission)

    #################### Signals ####################

    def close_tab_button(self, widget, swindow):
        """
        Close a tab in the DocumentManager
        
        @param widget: Reference to the widget.
        @type widget: A widget object.
        
        @param swindow: A ScrolledWindow.
        @type swindow: A ScrolledWindow object.
        """
        self.close_tab(swindow)
    
    def change_tab(self, widget, page, page_num):
        """
        This method does some operations when tabs are changed.
        """
        previous_view = widget.get_active_view()
        if previous_view != None:
            previous_view.word_complete.stop_indexer()
            
        view = self.get_nth_page(page_num).get_child()
        
        view.word_complete.start_indexer()
        
        self.search_functions.view = view
        self.search_functions.buffer = view.buffer
        
        self.search_functions.cursor_mark = \
        self.search_functions.buffer.get_insert()
         
        title = view.get_file_uri()
        if title == "":
            file_name = constants.MESSAGE_0001
            direc = ""
        else:
            f_list = title.split("/")
            file_name = f_list.pop()
            direc = " (" + "/".join(f_list) + ")"
        
        self.main_window.set_title(str(file_name) + direc)
        begin = view.buffer.get_start_iter()
        end = view.buffer.get_end_iter()
        self.search_functions.update_matches(begin, end)
        
        pos = view.get_cursor_position()
        
        #Getting view language id
        language = view.buffer.get_language()
        
        if language != None:
            language_id = language.get_id()
            self.language_manager.change_mode(language_id)
        else:
            self.language_manager.change_mode(None)
            
        self.interface.statusbar.set_line_column(pos[0], pos[1])
        self.interface.statusbar.combobutton_encoding.set_label(view.encode)
        self.interface.statusbar.permission_label.set_text(view.permission)
        self.interface.statusbar.combobutton_tab_size.set_label(constants.MESSAGE_0026 + " " + str(view.get_tab_width()))
    
    def drag_motion(self, textview, context, x, y, time):
        """
        Handles callback when a drag operation begins.
        """
        if "text/uri-list" in context.targets:
            return True
        else:
            return False

    def drag_drop(self, textview, context, x, y, time):
        """
        Handles callback when a drop operation begins.
        """
        if "text/uri-list" in context.targets:
            return True
        else:
            return False
        
    def drag_data_received(self, textview, context, xcord, ycord,
                              selection_data, info, timestamp):
        """
        Handles callback when a drop operation finishes.
        """
        
        if not ("text/uri-list" in context.targets):
            return False
        else:
            if info != 1:
                return False
            
            # Load file
            uri_list = list(selection_data.get_uris())
            
            for i in uri_list:
                filepatch = gnomevfs.get_local_path_from_uri(i)
                if os.path.getsize(filepatch) > 5120000:
                    dialog = gtk.MessageDialog(self.main_window, gtk.DIALOG_MODAL, gtk.MESSAGE_QUESTION, gtk.BUTTONS_YES_NO, (constants.MESSAGE_0030) % filepatch)
                    choice = dialog.run()
                    dialog.destroy()                    
                    if choice == gtk.RESPONSE_YES:
                        self.open_file_tab(filepatch)
                else:
                    self.open_file_tab(filepatch)
            
            context.finish(True, False, timestamp)
            return True                         
    
    def cursor_moved(self, buffer, position, view, statusbar):
        pos = view.get_cursor_position()
        statusbar.set_line_column(pos[0], pos[1])
    
    def key_release(self, widget, event):
        """
        This method does some operations when a key is realeased.
        """
        last = self.trigger_manager.last_shortcut
#         print "--------------------------"
        #print event.state
        #print event.keyval
#         print "--------------------------"
        if last in self.trigger_manager.sticky_keys:
            if event.state & gtk.gdk.CONTROL_MASK and event.state & gtk.gdk.SHIFT_MASK:
                pass
#                 print "ctrl+shift+" + unichr(event.keyval).lower()
            elif event.state & gtk.gdk.CONTROL_MASK:
                if "ctrl" in last and (event.keyval == 65507 or not (unichr(event.keyval) in self.trigger_manager.sticky_keys[last])):
                    try:
                        self.trigger_manager.sticky_shortcuts[last]()
                    #except KeyError:
                    #    return
                    finally:
                        self.trigger_manager.last_shortcut = None
                        self.doing_sticky = False
    #         elif event.state & gtk.gdk.SHIFT_MASK:
    #             shortcut = "shift+" + unichr(event.keyval).lower()
    #             shortcut2 = unichr(event.keyval).lower()
    #             if shortcut in self.trigger_manager.sticky_shortcuts:
    #                 self.trigger_manager.sticky_shortcuts[shortcut]()
    #             elif shortcut2 in self.trigger_manager.sticky_shortcuts:
    #                 self.trigger_manager.sticky_shortcuts[shortcut2]()
            else:
                shortcut = unichr(event.keyval).lower()
                last_key = last.split("+")[-1]
                if shortcut == last_key:
                    try:
                        self.trigger_manager.sticky_shortcuts[last]()
                    #except KeyError:
                    #    return
                    finally:
                        self.trigger_manager.last_shortcut = None
        
    def pre_key_press(self, widget, event):
        #print event.keyval
        #print event.state
        
        if self.doing_sticky:
            return False
        
        if event.state & gtk.gdk.CONTROL_MASK \
        and event.state & gtk.gdk.SHIFT_MASK:
            shortcut = "ctrl+shift+" + unichr(event.keyval).lower()
            if shortcut in self.trigger_manager.shortcuts:
                r = self.trigger_manager.shortcuts[shortcut]()
                self.trigger_manager.last_shortcut = shortcut
                
                if shortcut in self.trigger_manager.sticky_shortcuts:
                    self.doing_sticky = True
                
                return r
            elif shortcut in self.trigger_manager.language_shortcuts:
                r = self.trigger_manager.language_shortcuts[shortcut]()
                self.trigger_manager.last_shortcut = shortcut
                
                if shortcut in self.trigger_manager.sticky_shortcuts:
                    self.doing_sticky = True
                
                return r
            else:
                return False
        elif event.state & gtk.gdk.CONTROL_MASK:
            shortcut = "ctrl+" + unichr(event.keyval).lower()
            if shortcut in self.trigger_manager.shortcuts:
                r = self.trigger_manager.shortcuts[shortcut]()
                self.trigger_manager.last_shortcut = shortcut
                
                if shortcut in self.trigger_manager.sticky_shortcuts:
                    self.doing_sticky = True
                
                return r
            elif shortcut in self.trigger_manager.language_shortcuts:
                r = self.trigger_manager.language_shortcuts[shortcut]()
                self.trigger_manager.last_shortcut = shortcut
                
                if shortcut in self.trigger_manager.sticky_shortcuts:
                    self.doing_sticky = True
                
                return r
            else:
                return False
        elif event.state & gtk.gdk.SHIFT_MASK:
            shortcut = "shift+" + unichr(event.keyval).lower()
            shortcut2 = unichr(event.keyval).lower()
            if shortcut in self.trigger_manager.shortcuts:
                r = self.trigger_manager.shortcuts[shortcut]()
                self.trigger_manager.last_shortcut = shortcut
                
                if shortcut in self.trigger_manager.sticky_shortcuts:
                    self.doing_sticky = True
                
                return r
            elif shortcut2 in self.trigger_manager.shortcuts:
                r = self.trigger_manager.shortcuts[shortcut2]()
                self.trigger_manager.last_shortcut = shortcut2
                
                if shortcut in self.trigger_manager.sticky_shortcuts:
                    self.doing_sticky = True
                
                return r
            elif shortcut in self.trigger_manager.language_shortcuts:
                r = self.trigger_manager.language_shortcuts[shortcut]()
                self.trigger_manager.last_shortcut = shortcut
                
                if shortcut in self.trigger_manager.sticky_shortcuts:
                    self.doing_sticky = True
                
                return r
            elif shortcut2 in self.trigger_manager.language_shortcuts:
                r = self.trigger_manager.language_shortcuts[shortcut2]()
                self.trigger_manager.last_shortcut = shortcut2
                
                if shortcut in self.trigger_manager.sticky_shortcuts:
                    self.doing_sticky = True
                
                return r
            else:
                return False
        else:
            
            #print self.trigger_manager.language_shortcuts
            shortcut = unichr(event.keyval).lower()
                        
            if shortcut in self.trigger_manager.shortcuts:
                r = self.trigger_manager.shortcuts[shortcut]()
                self.trigger_manager.last_shortcut = shortcut
                
                if shortcut in self.trigger_manager.sticky_shortcuts:
                    self.doing_sticky = True
                
                return r
            elif shortcut in self.trigger_manager.language_shortcuts:
                r = self.trigger_manager.language_shortcuts[shortcut]()
                self.trigger_manager.last_shortcut = shortcut
                
                if shortcut in self.trigger_manager.sticky_shortcuts:
                    self.doing_sticky = True
                
                return r
            else:
                return False
        
    def buffer_change(self, widget, view, label):
        if not view.modified:
            text = label.get_text()
            view.modified = True
            label.set_text("*" + text)
        
        self.search_functions.all_search = False
        start, end = view.get_line_iters()
        self.search_functions.update_matches(start, end)
